"""
.. module:: mutprob.py
    :synopsis: Each mutation can increse the number of non-zero
    coefficients in the QGA. That increses the complexity of the
    simulation and the time needed. In this script we analyse
    the probability to have a different number of acumulated
    mutations in G generations, M, considering different pm.
.. moduleauthor::  Ruben Ibarrondo (rubenibarrondo@gmail.com)
"""

import numpy as np
import matplotlib.pyplot as plt

N = 8
cl = 6
G = 20
T = 200

def mutprob(M, pm):
    return np.math.factorial(N*cl)/np.math.factorial(M)/np.math.factorial(N*cl-M) * pm**M * (1-pm)**(N*cl-M)


def mutprob_gen(M, pm):
    return 1 - (1 - mutprob(M, pm))**(G*T)


x = np.arange(0, N*cl)
param_arr = np.arange(0.01, 0.06, 0.01)
param_name = '$p_m$'

for param in param_arr:
    y = [np.log10(mutprob_gen(M, param)) for M in x]
    plt.plot(x, y, label=param_name+' = {}'.format(param))

plt.xlim([-0.1, 20])
plt.ylim([-10, 1])

plt.title('Probability of $M$ mutations')
plt.xlabel('$M$')
plt.ylabel('$Probability$')
plt.grid(True)
plt.legend()

plt.savefig('Probability of $M$ mutations.png')
plt.show()

print('With pm = 0.01')
print('In 20*200 cases')
for M in range(30):
    print('{:3}'.format(M), mutprob(M, 0.01),  mutprob_gen(M, 0.01))

